@@ -20,8 +20,8 @@
 
 #include "BShaprGUI.hpp"
 
-BShaprGUI::BShaprGUI (const char *bundlePath, const LV2_Feature *const *features, PuglNativeWindow parentWindow) :
-	Window (1200, 710, "B.Shapr", parentWindow, true),
+BShaprGUI::BShaprGUI (const char *bundlePath, const LV2_Feature *const *features, PuglNativeView parentWindow) :
+	Window (1200, 710, "B.Shapr", parentWindow, true, PUGL_MODULE, 0),
 	controller (NULL), write_function (NULL),
 	bpm (120), beatsPerBar (4.0), beatUnit (4),
 
@@ -1688,11 +1688,11 @@ void BShaprGUI::updateHorizon ()
 	monitorHorizon2.moveLineTo ((horizonPos - 1) * width, 0);
 }
 
-LV2UI_Handle instantiate (const LV2UI_Descriptor *descriptor, const char *plugin_uri, const char *bundle_path,
+static LV2UI_Handle instantiate (const LV2UI_Descriptor *descriptor, const char *plugin_uri, const char *bundle_path,
 						  LV2UI_Write_Function write_function, LV2UI_Controller controller, LV2UI_Widget *widget,
 						  const LV2_Feature *const *features)
 {
-	PuglNativeWindow parentWindow = 0;
+	PuglNativeView parentWindow = 0;
 	LV2UI_Resize* resize = NULL;
 
 	if (strcmp(plugin_uri, BSHAPR_URI) != 0)
@@ -1703,7 +1703,7 @@ LV2UI_Handle instantiate (const LV2UI_Descriptor *descriptor, const char *plugin
 
 	for (int i = 0; features[i]; ++i)
 	{
-		if (!strcmp(features[i]->URI, LV2_UI__parent)) parentWindow = (PuglNativeWindow) features[i]->data;
+		if (!strcmp(features[i]->URI, LV2_UI__parent)) parentWindow = (PuglNativeView) features[i]->data;
 		else if (!strcmp(features[i]->URI, LV2_UI__resize)) resize = (LV2UI_Resize*)features[i]->data;
 	}
 	if (parentWindow == 0) std::cerr << "BShapr.lv2#GUI: No parent window.\n";
@@ -1734,35 +1734,47 @@ LV2UI_Handle instantiate (const LV2UI_Descriptor *descriptor, const char *plugin
 	return (LV2UI_Handle) ui;
 }
 
-void cleanup(LV2UI_Handle ui)
+static void cleanup(LV2UI_Handle ui)
 {
 	BShaprGUI* pluginGui = (BShaprGUI*) ui;
-	delete pluginGui;
+	if (pluginGui) delete pluginGui;
 }
 
-void portEvent(LV2UI_Handle ui, uint32_t port_index, uint32_t buffer_size,
+static void portEvent(LV2UI_Handle ui, uint32_t port_index, uint32_t buffer_size,
 	uint32_t format, const void* buffer)
 {
 	BShaprGUI* pluginGui = (BShaprGUI*) ui;
-	pluginGui->portEvent(port_index, buffer_size, format, buffer);
+	if (pluginGui) pluginGui->portEvent(port_index, buffer_size, format, buffer);
 }
 
 static int callIdle (LV2UI_Handle ui)
 {
 	BShaprGUI* pluginGui = (BShaprGUI*) ui;
-	pluginGui->handleEvents ();
+	if (pluginGui) pluginGui->handleEvents ();
 	return 0;
 }
 
-static const LV2UI_Idle_Interface idle = { callIdle };
+static int callResize (LV2UI_Handle ui, int width, int height)
+{
+	BShaprGUI* self = (BShaprGUI*) ui;
+	if (!self) return 0;
+	
+	BEvents::ExposeEvent* ev = new BEvents::ExposeEvent (self, self, BEvents::CONFIGURE_REQUEST_EVENT, self->getPosition().x, self->getPosition().y, width, height);
+	self->addEventToQueue (ev);
+	return 0;
+}
+
+static const LV2UI_Idle_Interface idle = {callIdle};
+static const LV2UI_Resize resize = {nullptr, callResize};
 
 static const void* extensionData(const char* uri)
 {
 	if (!strcmp(uri, LV2_UI__idleInterface)) return &idle;
+	else if(!strcmp(uri, LV2_UI__resize)) return &resize;
 	else return NULL;
 }
 
-const LV2UI_Descriptor guiDescriptor = {
+static const LV2UI_Descriptor guiDescriptor = {
 		BSHAPR_GUI_URI,
 		instantiate,
 		cleanup,
